@@ -1,5 +1,7 @@
+use crate::error::TryFromError;
+
 use std::fmt;
-use std::convert::TryInto;
+use std::convert::{TryFrom, TryInto};
 
 use generic_array::{GenericArray, typenum};
 use typenum::{U16};
@@ -24,23 +26,10 @@ impl Mac {
 		Self { tag }
 	}
 
-	/// This function should only be used with bytes that
-	/// were received with a message.
-	pub fn from_bytes(bytes: [u8; 16]) -> Self {
-		let gen: GenericArray<u8, U16> = bytes.into();
-
-		Self { tag: Tag::new(gen) }
-	}
-
 	/// ## Panics
 	/// if the slice is not 16 bytes long.
 	pub fn from_slice(slice: &[u8]) -> Self {
-		Self::from_bytes(slice.try_into().unwrap())
-	}
-
-	pub fn try_from_slice(slice: &[u8]) -> Option<Self> {
-		slice.try_into().ok()
-			.map(Self::from_bytes)
+		slice.try_into().unwrap()
 	}
 
 	pub fn into_bytes(self) -> [u8; 16] {
@@ -55,7 +44,21 @@ impl fmt::Debug for Mac {
 }
 
 impl From<[u8; 16]> for Mac {
+	/// This function should only be used with bytes that
+	/// were received with a message.
 	fn from(bytes: [u8; 16]) -> Self {
-		Self::from_bytes(bytes)
+		let gen: GenericArray<u8, U16> = bytes.into();
+
+		Self { tag: Tag::new(gen) }
 	}
 }
+
+impl TryFrom<&[u8]> for Mac {
+	type Error = TryFromError;
+
+	fn try_from(s: &[u8]) -> Result<Self, Self::Error> {
+		<[u8; 16]>::try_from(s)
+			.map_err(TryFromError::from_any)
+			.map(Mac::from)
+	}
+}
\ No newline at end of file
