@@ -1,6 +1,7 @@
 use crate::fill_random;
+use crate::error::TryFromError;
 
-use std::convert::TryInto;
+use std::convert::{TryFrom, TryInto};
 
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub struct Nonce {
@@ -27,20 +28,10 @@ impl Nonce {
 		Self { bytes: [1u8; 24] }
 	}
 
-	/// Creates a nonce from bytes.
-	pub fn from_bytes(bytes: [u8; 24]) -> Self {
-		Self { bytes }
-	}
-
 	/// ## Panics
 	/// if the slice is not 24 bytes long.
 	pub fn from_slice(slice: &[u8]) -> Self {
-		Self::from_bytes(slice.try_into().unwrap())
-	}
-
-	pub fn try_from_slice(slice: &[u8]) -> Option<Self> {
-		slice.try_into().ok()
-			.map(Self::from_bytes)
+		slice.try_into().unwrap()
 	}
 
 	pub fn to_bytes(&self) -> [u8; 24] {
@@ -52,10 +43,6 @@ impl Nonce {
 		self.bytes
 	}
 
-	pub fn as_slice(&self) -> &[u8] {
-		&self.bytes
-	}
-
 	/// Takes the current nonce, replacing it
 	/// with a new random one.
 	pub fn take(&mut self) -> Self {
@@ -64,3 +51,26 @@ impl Nonce {
 	}
 
 }
+
+impl From<[u8; 24]> for Nonce {
+	/// Creates a nonce from bytes.
+	fn from(bytes: [u8; 24]) -> Self {
+		Self { bytes }
+	}
+}
+
+impl TryFrom<&[u8]> for Nonce {
+	type Error = TryFromError;
+
+	fn try_from(s: &[u8]) -> Result<Self, Self::Error> {
+		<[u8; 24]>::try_from(s)
+			.map_err(TryFromError::from_any)
+			.map(Nonce::from)
+	}
+}
+
+impl AsRef<[u8]> for Nonce {
+	fn as_ref(&self) -> &[u8] {
+		&self.bytes
+	}
+}
\ No newline at end of file
